! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
! +++ Program: B2DWAVE.F90 +++                +++  T.H.C. Herbers ++
!                                              +++  and            ++
! 11/14/00                                    +++  Mark Orzech    ++
! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! The 2-d Boussinesq equations for a gently sloping beach with straight
! and parallel depth contours are solved for a continuous spectrum of 
! waves.  Evolution equations for the frequency/alongshore wavenumber 
! spectrum and bispectrum ((22a,b) from Herbers and Burton, J. Geophys. 
! Res., 102(C9), 21101-21114, 1997) are integrated using a fourth-order 
! Runge Kutta scheme. Inputs provided by the user are: the discretized 
! bottom profile, the frequency/alongshore wavenumber grid, and the 
! initial spectrum.  The initial bispectrum is evaluated based on the 
! steady flat bottom solution (23).  Symmetry relations are used to 
! reduce the integrations to the non-redundant region of the interaction 
! space. 
!
! Recommended compiling syntax on Linux PCs running Intel 
! Fortran 90 Compiler (V 7.0):
!
!            ifc -tpp7 -xW -132 -ob2dwave b2dwave.f90
!
! or (for V 8.0, which requires extra info to set default output record length unit to 1 byte):
!
!            ifort -tpp7 -xW -assume byterecl -132 -ob2dwave b2dwave.f90
!
! Recommended alternate compiling syntax on various unix/linux workstations:
!
!            f90 -O3 -ob2dwave b2dwave.f90
!
! Reference publications used below:
!
! Ref 1: Herbers et al., "Shoaling transformation of wave frequency-directional spectra",
!        J. of Geophysical Research, Vol 108, No. C1, 3013, 2003.
! Ref 2: Herbers & Burton, "Nonlinear shoaling of directionally spread waves on a beach",
!        J. of Geophysical Research, Vol 102, No. C9, Sep 15 1997.
!
! Parameters:
!
!        alpha                                dissipation coefficient (see "getalpha")
!        asym(1:nx)                        wave asymmetry
!        b,gam                                adjustable parameters B & gamma in Chen-
!                                          Kirby dissipation eqns; see "getalpha"
!        birand                                bispectral randomization factor used
!                                          to increase dissipation (see "calceb")
!        bs(1:3,1:ntriad)*                bispectrum (m^5 s^2/rad^4)
!                                          (1=previous,2=temporary,3=final)
!        c(1:nx)                                shallow water wave speed (m/s)
!        cbs(1:nsg,1:nsg)*                dissipation coefficient, bispectrum ((rad/s)^gexp)
!        cd(1:ntriad)*                        dispersion coefficient of Eq. (22b)
!        ces(1:nsg)*                        dissipation coefficient, spectrum ((rad/s)^gexp)
!        cn(1:nsg,1:nx)*                        nonlinear coefficient of Eqs. (22a,b)
!        cs1(1:nx)                        shoaling coefficient of Eq. (22a)
!        cs2(1:nx)                        shoaling coefficient of Eq. (22b)
!        dflag                                flag to turn on/off linear dissipation
!        dsg                                frequency spacing (rad/s)
!        dwny                                alongshore wavenumber spacing (rad/m)
!        dx                                spatial resolution (m)
!        es(1:3,1:nsg,-nwny:nwny)*        spectrum (m^3 s/rad^2)
!                                          (1=previous,2=temporary,3=final)
!        flin                                initial energy flux (m^3/s)
!        flux                                local energy flux (m^3/s)
!        g                                gravitational constant (m/s^2)
!        gb                                bispectrum gradient estimate
!        ge(1:nsg,-nwny:nwny)                spectrum gradient estimate
!        gexp                                dissipation exponent for 
!                                          calculation of cbs and ces coefficients
!        h(1:nx)                                depth profile (m)
!        hsig                                significant wave height (m)
!        i1max, i2max(1:i1max)*                maximum frequency indices
!        iit                                Runge-Kutta iteration step (1-4)
!        igr,iev,fstep,tstep                step sizes in Runge-Kutta iteration,
!                                          assigned by subroutine "setstep"
!        j1min, j1max,                        minimum/maximum alongshore
!           j2min(j1min:j1max)*,                  wavenumber indices
!           j2max(j1min:j1max)*
!        nsg                                number of positive frequencies
!        ntriad                                number of triads in the model domain
!        nwny                                number of positive alongshore 
!                                                wavenumbers
!        nx                                number of spatial grid points
!        sg(1:nsg)*                        frequencies (rad/s)
!        skew(1:nx)                        wave skewness 
!        specflag                        flag to specify type of spectral input
!        var                                total variance (m^2)
!        wny(-nwny:nwny)*                alongshore wavenumbers (rad/m)
!        
!      *NOTE:  Asterisk (*) implies variables are declared in module b2dvars
!
!        KEY TO OUTPUT FILE PREFIXES:
!
!        b2d.outputA:  (ASCII) Values of nsg,nwny,nx,dx, then all values of
!                        sg, wny, h
!        b2d.outputB:  (ASCII) Skewness and asymmetry at initial and nloc 
!                        locations only
!        b2d.outputC:  (binary) Spectrum es(1,i,j), at initial and nloc 
!                        locations only
!        b2d.outputD:  (ASCII) x,h,IG var,hsig,skew,asym at all model 
!                        steps (6 columns)

! **************************************************************

MODULE b2dvars

!
! Create set of allocatable arrays and matrices
! to be used in b2d.f90 as "common block" variables
!
        real, dimension(:), allocatable :: cd,ces
        complex, dimension(:,:), allocatable :: bs,cbs

        real, dimension(:,:,:), allocatable :: es
        real, dimension(:,:), allocatable :: cn

        real, dimension(:), allocatable :: sg,wny
        integer, dimension(:), allocatable :: i2max,j2min,j2max
        
        integer nsg,nwny

END MODULE b2dvars

! **************************************************************


PROGRAM b2d

        use b2dvars
        
        dimension h(10001),xall(10001),hall(10001),varall(10001)
        dimension hsall(10001),skall(10001),asall(10001)
        dimension c(10001),cs1(10001),cs2(10001)
        complex xim
        
! **************************************************************
!        
        
        real gexp,alpha,gam,b,var,vari
        real skew,asym,sgmin,sgmax,birand,MAX
        integer dflag,depflag,specflag,nx,recno
        integer nlocs,iloc,reclen
        integer reclen2,reclen3,nloc(150)
        character*6 fdate
        character*17 outA,outB,outC,outD
        logical isnan
        
!
! **************************************************************

        g=9.806194
        pi=3.14159265358979
        xim=(0.0,1.0)
        recno = 0
        MAX = 9999E9;

! **************************************************************
!
!        obtain depth profile and calculate shoaling coefficients
!
        call loaddep(dx,nx,h,c,cs1,cs2)
!
! **************************************************************
!
!       obtain initial spectrum and frequency/wavenumber information
!
        call loadspec(dsg,dwny,ntriad,sgmin,sgmax)
!
! ***************************************************************
!
!       allocate space for energy and bispectrum calculation terms
!
        call setgrid(ntriad)
!        
! ***************************************************************
!
!        Obtain values selecting which data locations to save and file date info

        print *,'enter file ID string (up to 6 char): '
        read (*,*) fdate
        print *,'enter number of locations to save spectra: '
        read (*,*) nlocs
        print *,nx,' grid points; select indices of ', nlocs,' points: '
        read (*,*) (nloc(i), i=1,nlocs)
        print *,(nloc(i), i=1,nlocs)
        print *,nsg,' frequencies: ',sg(1),'-',sg(nsg),' rad/s'
        print *,2*nwny+1,' alongshore wavenumbers: ', &
                     wny(-nwny),'-',wny(nwny),' rad/m'

! *************************************************************
!        Assign output file names

        outA='b2d.outputA'//fdate
        outB='b2d.outputB'//fdate
        outC='b2d.outputC'//fdate
        outD='b2d.outputD'//fdate
        
! *************************************************************
!        Set values for dissipation parameters, if applicable
        
        call getdis(dflag,gam,b,birand,gexp)
        
!
! *************************************************************

!
!        Write the model discretization, bottom profile, to one output file 
!        and initial spectrum to another output file.
!        Also open additional binary/ascii output files.
!
        open (unit=16,file=outA)
        write (16,*) nsg,nwny,nx,dx
        write (16,*) (sg(i), i=1,nsg)
        write (16,*) (wny(i), i=-nwny,nwny)
        write (16,*) (h(i), i=1,nx)
        close (16)

        open (unit=18,file=outB)
        reclen = (2*nwny + 1)*nsg*4
        open(unit=30,file=outC,form='unformatted', &
         access='direct',recl=reclen)
        
!
!        compute the ranges of the four indices of the bispectrum
!
      call indices(i1max,j1min,j1max)

!  ********************************************************************
!
!     compute coefficients for dissipation in bispectrum & energy eqns
!
      call discoef(dflag,i1max,gexp)
!
!  ********************************************************************

!
!       compute nonlinear interaction and dispersion coefficients,
!       and evaluate the initial bispectrum
!
        call calcbi(nx,h,c,i1max,j1min,j1max,ntriad)
        
! *****************************************************************
!        
!       calculate initial Hs, skewness and asymmetry
!
        call calchsa(i1max,j1min,j1max,dsg,dwny,        &
                     iev,skew,asym,sgmin,sgmax,hsig,vari,var)
        print *,'Hs = ',hsig,' skewness = ',skew,'; asymm = ',asym

!       Calculate initial energy flux 
        flin=(vari+var)*c(1)
        
!       save initial skewness and asymmetry values to the 
!       file 'b2d.outputB'
!
        write (18,*) skew, asym
!        
! *****************************************************************
!
!        integrate the spectrum and bispectrum evolution equations
!        with a 4th order Runge-Kutta scheme; check energy conservation
!        and write the spectrum estimate to the output file
!
        do 390 ix=1,nx-2,2

           do 250 iit=1,4
!
!        set step sizes for multipliers used in Runge-Kutta calculations
!
                call setstep(iit,igr,ix,iev,fstep,dx,tstep)
!        
!        compute dissipation coefficient alpha
!
                if (dflag.eq.1) then
                   call getalpha(dsg,h,gexp,iev,alpha,gam,b,ix,dwny)
                else
                   alpha = 0
                endif
!
!        compute energy spectrum, es, and bispectrum, bs, then wave statistics
!
                      call calceb(alpha,igr,iev,c,cs1,cs2,   &
                              iit,i1max,j1min,j1max,dsg,dwny,  &
                              fstep,tstep,ntriad,birand)


                call calchsa(i1max,j1min,j1max,dsg,dwny,  &
                         iev,skew,asym,sgmin,sgmax,hsig,vari,var)

!        Make sure we haven't got NaNs; if we do, quit program now 

            if (var.lt.MAX.and.vari.lt.MAX.and.hsig.ge.0.0) then
              isnan=.FALSE. 
            else
              isnan=.TRUE. 
              print *,'B2DWAVE UNSTABLE:' 
              print *,'var=',var,' vari=',vari,' hsig=',hsig
              go to 400
            end if
        
250       end do

297       flux=(vari+var)*c(ix+2)
!
!        Write step number to screen
!         
           print *,'STEP ',(ix+1)/2,        ' OF ',(nx-1)/2, ': '
                 print *,'  h = ',h(ix+2),' m, ', &
               '  relative energy flux = ',flux/flin        

!        Write x, h, Hs, skew, asym at each step to vectors 

           xall((ix+1)/2) = float(ix+1)*dx
           hall((ix+1)/2) = h(ix+2)
           varall((ix+1)/2) = vari
           hsall((ix+1)/2) = hsig
           skall((ix+1)/2) = skew
           asall((ix+1)/2) = asym
           
           
!        If at user-specified location on profile, write results to output files

           do 300 iloc=1,nlocs
              if ((ix).eq.nloc(iloc).or.(ix+1).eq.nloc(iloc)) then
                 recno=recno+1
                 call writedat(recno,h,hsig,dx,ix,dwny,skew,asym,iloc)
              end if
300        end do
           go to 390
        
390      end do

!        When all loops are finished, write all x,h,IG var,Hs,skew,asym data
!        to a single output file in 5 columns (matlab loadable)

        open (38,file=outD)
        write (38,*) '% x, h, Infragrav var, Hs, skew, asym'
        do 395 ix=1,((nx-1)/2)
           write (38,397) xall(ix),hall(ix),varall(ix),hsall(ix), &
              skall(ix),asall(ix)
395     end do
397     format(f8.2,1x,f7.4,1x,e15.6,1x,f10.4,1x,e15.6,1x,e15.6)
        close(38)           
        
400     close (18)
        close (30)

1000    stop
        end

! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!
!                        SUBROUTINES
!

!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!        subroutine calcbi
!
!        compute nonlinear interaction and dispersion coefficients,
!        and evaluate the initial bispectrum
!        
!        Local Parameters:  none
!        

      subroutine calcbi(nx,h,c,i1max,j1min,j1max,ntriad)
     
        use b2dvars
        
        integer ix,nx,isg,itriad,ntriad,i1max,j1min,j1max
        integer i1,j1
        dimension h(10001),c(10001)
        
        g=9.806194
        
        do 1104 ix=1,nx
                do 1102 isg=1,nsg
        cn(isg,ix)=(3.0*sg(isg))/(2.0*h(ix)*c(ix))
1102            continue
1104    continue

        itriad=0
        do 1109 i1=1,i1max,1
                do 1108 i2=1,i2max(i1)
                        do 1107 j1=j1min,j1max
                                do 1106 j2=j2min(j1),j2max(j1)
        itriad=itriad+1
        cd(itriad)=(sg(i1)*sg(i2)*sg(i1+i2))/(2.0*g**2)        &
                      +(sg(i1+i2)*wny(j1)-sg(i1)*wny(j1+j2))**2        &
                      /(2.0*sg(i1)*sg(i2)*sg(i1+i2))
        bs(1,itriad)=-(cn(i1,1)*es(1,i2,j2)*es(1,i1+i2,j1+j2)        &
                      +cn(i2,1)*es(1,i1,j1)*es(1,i1+i2,j1+j2)        &
                      -cn(i1+i2,1)*es(1,i1,j1)*es(1,i2,j2))        &
                      /(c(1)*cd(itriad))
        bs(3,itriad)=bs(1,itriad)
1106                            continue
1107                    continue
1108            continue
1109    continue
        
        if (itriad .ne. ntriad) then
           print *,itriad,' ERROR detected in triad index!'
           stop
        end if
        
        return
        end

!
!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++




!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!        subroutine calceb
!
!        calculates new es and bs values in Runge Kutta iteration process
!        NB:  Uses "birand" factor to increase randomization of bispectrum
!                in calculations of bispectral gradient (gb) below
!        
!        Local Parameters:  
!        
!        dd                product of dsg and dwny (*2 if required by symmetries)
!        tr1,tr2                bispectral terms used in calculation of gradient of
!                          energy spectrum (Ref 1, second term on rhs of eqn 5a)
!        xim                imaginary number i, sqrt(-1)
!        

      subroutine calceb(alpha,igr,iev,c,cs1,cs2,  &
                              iit,i1max,j1min,j1max,dsg,dwny,        &
                              fstep,tstep,ntriad,birand)
     
        use b2dvars
      
        integer igr,iev,isg,iwny,itriad,ntriad
        integer i1max,iit
        integer j1min,j1max
        integer i1,j1,i2,j2
        real dsg,dwny,dd,fstep,tstep,alpha
        real birand,tr1,tr2
        dimension c(10001),cs1(10001),cs2(10001)
        dimension ge(nsg,-nwny:nwny)
        complex gb
        complex xim
        
        xim=(0.0,1.0)

        do 1309 isg=1,nsg
           do 1308 iwny=-nwny,nwny
                        
!             Calculate gradient of energy spectrum (See Ref 1, eq's 5a & 6a)
              ge(isg,iwny)=(cs1(igr)-(alpha*ces(isg)))*es(iev,isg,iwny)

1308       continue
1309    continue
        itriad=0
        do 1314 i1=1,i1max
                
           do 1313 i2=1,i2max(i1)
                if (i1 .eq. i2) then
                        dd=dsg*dwny
                else
                        dd=2.0*dsg*dwny
                end if
                do 1312 j1=j1min,j1max
                   do 1311 j2=j2min(j1),j2max(j1)
                itriad=itriad+1
                tr1=cn(i1,igr)*aimag(bs(iev,itriad))*dd
                tr2=cn(i2,igr)*aimag(bs(iev,itriad))*dd

!              Add bispectral triad modifications to energy spectrum gradient
                ge(i1+i2,j1+j2)=ge(i1+i2,j1+j2)+tr1+tr2
                ge(i1,j1)=ge(i1,j1)-tr1
                ge(i2,j2)=ge(i2,j2)-tr2
        
!    Calculate gradient of bispectrum (Ref 1, eq's 5b & 6b)
      gb=(cs2(igr) -birand*alpha*cbs(i1,i2) -xim*c(igr)*cd(itriad))*        &
                      bs(iev,itriad)-xim*        &
                      (cn(i1,igr)*es(iev,i2,j2)*es(iev,i1+i2,j1+j2)        &
                      +cn(i2,igr)*es(iev,i1,j1)*es(iev,i1+i2,j1+j2)        &
                      -cn(i1+i2,igr)*es(iev,i1,j1)*es(iev,i2,j2))        
     
                bs(3,itriad)=bs(3,itriad)+gb*fstep
                if (iit .le. 3) then
                   bs(2,itriad)=bs(1,itriad)+gb*tstep
                else
                   bs(1,itriad)=bs(3,itriad)
                end if

1311               continue
1312           continue
1313       continue
1314    continue
        if (itriad .ne. ntriad) then
                   print *,itriad,        &
                         ' calceb: ERROR detected in triad index!'
                   stop
        end if
                
!      Update energy spectrum as appropriate for Runge-Kutta iteration step, iit          
        do 1322 isg=1,nsg
                do 1321 iwny=-nwny,nwny
        es(3,isg,iwny)=es(3,isg,iwny)+ge(isg,iwny)*fstep
        if (iit .le. 3) then
           es(2,isg,iwny)=es(1,isg,iwny)+ge(isg,iwny)*tstep
        else
           es(1,isg,iwny)=es(3,isg,iwny)
        end if
1321            continue
1322    continue

        return
        end
!
!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        



!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!        subroutine calchsa
!
!        calculate significant wave height (m) from energy spectrum, es, 
!        and skewness and asymmetry of free surface from bispectrum, bs
!        
!        Local Parameters:  
!        
!        m3                third moment of free surface (m^3)
!        dsw2                product of differential elements used in obtaining 
!                          skewness and asymmetry (rad^4/(m^2 s^2))
!        dd                multiplicative factor required to account for
!                          symmetries in bispectral calculations
!        

      subroutine calchsa(i1max,j1min,j1max,dsg,dwny,        &
                         iev,skew,asym,sgmin,sgmax,hsig,vari,var)
        
        use b2dvars, only: bs,es,i2max,j2min,j2max,nsg,nwny
        
        integer itriad,isg
        integer i1max
        integer j1min,j1max
        integer i1,j1,i2,j2
        real dsg,dwny,dsw2,dd,skew,asym,pi,hsig
        real sgmin,sgmax,vari,var
        complex m3
        
        m3=(0.0,0.0)
        itriad=0
        pi=3.14159265358979
        dsw2=dsg*dsg*dwny*dwny
!
!        Calculate statistics in range sgmin --> sgmax
!        (Note:  You only need to check "i2<sgmin" for skew and asym
!               below, because i2<=i1 everywhere.
!               Similarly, only need to check "i1>sgmax" below, because
!               i1>=i2 everywhere.)
!

!        calculate sea-swell/infragrav variances and hsig from spectrum

        var=0.0
        vari=0.0
        do 1112 isg=1,nsg
          if ((float(isg)*dsg).ge.sgmin) then
            if ((float(isg)*dsg).le.sgmax) then
              do 1110 iwny=-nwny,nwny
                var=var+es(1,isg,iwny)
1110          continue
            endif
          else
            do 1111 iwny=-nwny,nwny
              vari=vari+es(1,isg,iwny)
1111        continue
          endif
1112    continue
        var=2.0*var*dsg*dwny
        vari=2.0*vari*dsg*dwny
        
        hsig=4.0*sqrt(var)

!        calculate skewness and asymmetry from bispectrum

        do 1118 i1=1,i1max
           do 1116 i2=1,i2max(i1)
           
                   if ((float(i2)*dsg).lt.sgmin) then
                            dd=0.0
                elseif ((float(i1)*dsg).gt.sgmax) then
                        dd=0.0
                   elseif (i1 .eq. i2) then
                        dd=6.0
                   else
                        dd=12.0
                   end if
                
                do 1114 j1=j1min,j1max
                   do 1113 j2=j2min(j1),j2max(j1)
                        itriad=itriad+1
                        m3=m3+bs(1,itriad)*dsw2*dd
1113               continue
1114            continue
1116       continue
1118    continue

        skew=real(m3)/(var**1.5)
        asym=aimag(m3)/(var**1.5)
        
        return
        end
!
! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++




!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!        subroutine discoef
!
!        compute coefficients for dissipation in bispectrum eqn
!        
!        Local Parameters:  none
!        

      subroutine discoef(dflag,i1max,gexp)
        
        use b2dvars, only: sg,i2max,cbs,ces,nsg
        
        integer dflag,i1max,i1,i2,isg
        real gexp
!
!        Generate bispectrum dissipation coefficients, cbs, if required
!                
        if (dflag.eq.1) then
           do 1125 i1 = 1,i1max
              do 1124 i2 = 1,i2max(i1)
                 cbs(i1,i2) = sg(i1)**gexp         &
                                      + sg(i2)**gexp         &
                                      + sg(i1+i2)**gexp
1124          continue
1125       continue
        else
           do 1127 i1 = 1,i1max
              do 1126 i2 = 1,i2max(i1)
                 cbs(i1,i2) = 0
1126          continue
1127       continue
        endif
!
!        Generate energy spectrum dissipation coefficients, ces
!        
        do 1150 isg = 1,nsg
           ces(isg) = 2*(sg(isg)**gexp)
1150    continue

        return
        end
!
!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        
        

!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!        subroutine getalpha
!
!        compute dissipation coefficient alpha
!
!       Calculate four main parts of alpha expression, then alpha.
!       For further information on "chen" terms in equations below,
!       see eqn 8 of Chen et al., "Modeling spectra of breaking surface waves
!         in shallow water", JGR, vol 102, No. C11, 1997.
!        To obtain alpha, an energy balance model is utilized in
!        which the right-hand side of eqn 8 is set equal to
!        the frequency integrated dissipation, given by alpha*disrate below.
!       (See also Ref 1, Section 2.2.) 
!        
!         Parameters:  
!        
!        alpha                1st coeff in linear damping term
!        b,gam                adjustable constants in Chen, eqn 8
!        chen1                first term on rhs of Chen, eqn 8
!        chen2                2nd term on rhs of Chen, eqn 8, in {}'s
!        chen3                3rd term on rhs of Chen, eqn 8, in {}'s
!        disrate*alpha        integral of dissipation from energy balance eqn
!        e1d                1D energy spectrum (m^2 s/rad)(summed over wavenumbers)
!        eint                integral of e1d*dsg (m^2)(to calculate fbar) 
!        fbar                weighted avg frequency (degrees) = sbar/2pi
!        g0                2nd coeff in linear damping term, = (sg)**gexp
!        gexp                exponent in linear damping term, g0 (also used elsewhere)
!        gint                integral of e1d*g0*dsg
!        rho                density of seawater (kg/m^3)
!        sbar                weighted avg frequency (radians) = sint/eint
!        sint                 sum of e1d*sg*dsg (to calculate fbar)
!
! ********************

      subroutine getalpha(dsg,h,gexp,iev,alpha,gam,b,ix,dwny)

        use b2dvars, only: es,sg,nsg,nwny
        
        integer iev,isg,iwny,ix
        real e1d,gexp,alpha,dsg
        real sint,eint,gint,fbar,sbar,h(10001)
        real chen1,chen2,chen3,disrate,gam,b,g0,hrms,rho,g,pi

        g=9.806194
        pi=3.14159265358979
        rho = 1026
        sint = 0.0
        gint = 0.0
        eint = 0.0

!       Calculate definite integrals
        
      do 1206 isg = 1,nsg
        e1d = 0.0
        do 1204 iwny = -nwny,nwny
           e1d = e1d + es(iev,isg,iwny)
1204    continue
        e1d = e1d*dwny
        g0 = (sg(isg))**gexp
        eint = eint + dsg*e1d
        sint = sint + (sg(isg))*dsg*e1d
        gint = gint + (dsg*g0*e1d)
1206  continue

      hrms = 2*sqrt(2.0)*sqrt(2.0*eint)
      sbar = sint/eint
      fbar = sbar/(2*pi)

!     "Chen" dissipation equation:
!     Calculate four parts of alpha expression, then alpha.

      chen1 = (3*sqrt(pi)/(16*h(ix)))*rho*g*b*fbar*(hrms**3)
      chen2 = 1+tanh(8*(hrms/(gam*h(ix))-1))
      chen3 = 1-((1+ ((hrms/(gam*h(ix)))**2) )**(-2.5))
      disrate = 4*rho*(g**1.5)*(h(ix)**0.5)*gint

      alpha = (chen1/disrate)*chen2*chen3

      return
      end
!
!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++




!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!        subroutine getdis
!
!        input values used in computation of dissipation, or set all 
!        values to zero if dissipation not applied
!
!        Local Parameters:  none
!        

      subroutine getdis(dflag,gam,b,birand,gexp)
      
        integer dflag
        real gam,b,birand,gexp

        print *,'dissipation flag (enter 1 to turn on): '
        read (*,*) dflag
        
        if (dflag.eq.1) then
           gam=0.3
           b=0.25
           birand=3.5
           gexp=2
        else
           gam=0
           b=0
           birand=0
           gexp=0
        endif

        return
        end

!        
!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++




!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!        subroutine indices
!
!        compute the ranges of the four indices of the bispectrum
!
!        Local Parameters:  none
!        

      subroutine indices(i1max,j1min,j1max)
        
        use b2dvars, only: i2max,j2min,j2max,nsg,nwny
        
        integer i1max
        integer j1min,j1max
        integer i1,j1,i,j
        
        print *,' min,max bispectrum indices: '
        i1max=nsg-1
        print *,' i1min=1; i1max=',i1max
        do 1091 i1=1,i1max
                i2max(i1)=nsg-i1
                if (i2max(i1) .gt. i1) i2max(i1)=i1
1091    continue
        print *,' i2min=1; i2max=',(i2max(i), i=1,i1max)
        j1min=-nwny
        j1max=nwny
        print *,' j1min=',j1min,'; j1max=',j1max
        do 1092 j1=j1min,j1max
                j2min(j1)=-nwny
                j2max(j1)=nwny
                if (j1 .lt. 0) j2min(j1)=-nwny-j1
                if (j1 .gt. 0) j2max(j1)=nwny-j1
1092    continue
        print *,' j2min=',(j2min(j), j=j1min,j1max)
        print *,' j2max=',(j2max(j), j=j1min,j1max)

        return
        end
!
!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++




! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!        subroutine loaddep
!
!       load depth data from file and compute shoaling term and wave speed
!
!        Local Parameters:
!        
!        nxr                (real) number of spatial grid points
!        slope                local slope of profile
!        flnm                input depth filename
!        

      subroutine loaddep(dx,nx,h,c,cs1,cs2)

        real nxr,dx,slope,h(10001),c(10001)
        real g,cs1(10001),cs2(10001)
        integer ix,nx
        character*25 flnm
        
        g=9.806194
        
        print *,'Enter depth file filename: '
        read (*,*) flnm
        open (unit=17,file=flnm,status='unknown')
        read (17,*) nxr
        nx = nxr
        read (17,*) dx
        read (17,*) h(1)
        do 1015 ix = 1,(nx-1)
                read(17,*) h(ix+1)
                slope = (h(ix) - h(ix+1))/dx
                c(ix)=sqrt(g*h(ix))
                cs1(ix)=slope/(2.0*h(ix))
                cs2(ix)=(3.0*slope)/(4.0*h(ix))
1015    continue

!       use preceding slope to calculate cs1(nx) and cs2(nx)

        c(nx) = sqrt(g*h(nx))
        cs1(nx) = slope/(2.0*h(nx))
        cs2(nx) = (3.0*slope)/(4.0*h(nx))
        close(17)

        return
        end
!
! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



        
! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!        subroutine loadspec
!
!        creates frequency and wavenumber vectors, calculates size of
!        triad interaction space, and loads spectrum (es) from specified file 
!        
!        Local Parameter:  
!        
!        fnm        name of spectral input file
!

      subroutine loadspec(dsg,dwny,ntriad,sgmin,sgmax)

        use b2dvars, only: es,nsg,nwny,sg,wny
        
        integer i,j,ntriad,isg
        character*30 fnm
        real dsg,dwny
        
        print *,'enter spectrum filename (max 30 char): '
        read (*,*) fnm
        open(unit=26,file=fnm,status='old')

        read (26,*) nsg
        read (26,*) dsg
!
! allocate sg array space in memory, then assign values
!
        allocate(sg(nsg))
        do 1040 isg=1,nsg
                sg(isg)=float(isg)*dsg
1040    continue

        read (26,*) nwny
        read (26,*) dwny

        if (nwny.eq.0) then
            dwny = 1.0
        endif

!
! allocate wny space in memory, then assign values
!
        allocate(wny(-nwny:nwny))
        do 1041 iwny=-nwny,nwny
            wny(iwny)=float(iwny)*dwny
1041    continue

        ntriad=((nsg**2)/4)*(3*nwny**2+3*nwny+1)
        print *,ntriad,' triads in model domain'

!
!       allocate energy spectrum space in memory, then read in values
!
        allocate(es(3,nsg,-nwny:nwny))
        if (nwny.eq.0) then
           read (26,*) (es(1,i,0), i=1,nsg)
        else
           read (26,*) ((es(1,i,j), j=-nwny,nwny), i=1,nsg)
        endif
        close(26)
        
        es(3,:,:)=es(1,:,:)
        
        print *,'Enter min, max freq (rad/s) for calculation of '
        print *,' Hs, skewness and asymmetry (eg 0.314 3.01): '
        read (*,*) sgmin, sgmax

        return
        end
!
! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++




! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!        subroutine setgrid
!
!        define the frequency and alongshore wavenumber discretization 
!        and compute the size of the triad interaction space
!        
!        Local Parameters:  none
!        
      
      subroutine setgrid(ntriad)
        
        use b2dvars
        
!
! allocate remaining memory space, then return
!
        allocate(cd(ntriad),i2max(nsg-1),ces(nsg))
        allocate(j2min(-nwny:nwny),j2max(-nwny:nwny))
        allocate(cn(nsg,10001),bs(3,ntriad),cbs(nsg,nsg))

        return
        end
!
! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++




!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!        subroutine setstep
!
!        set values of igr,iev,fstep,tstep based on iteration value, iit
!        
!        Local Parameters:  none
!        

      subroutine setstep(iit,igr,ix,iev,fstep,dx,tstep)
      
              integer iit,igr,ix,iev
        real fstep,dx,tstep

        if (iit .eq. 1) then
                igr=ix
                iev=1
                fstep=dx/3.0
                tstep=dx
        else if (iit .eq. 2) then
                igr=ix+1
                iev=2
                fstep=2.0*dx/3.0
                tstep=dx
        else if (iit .eq. 3) then
                igr=ix+1
                iev=2
                fstep=2.0*dx/3.0
                tstep=2.0*dx
        else if (iit .eq. 4) then
                igr=ix+2
                iev=2
                fstep=dx/3.0
        else
                print *,'setstep: iit out of range'
                stop
        end if
        
        return
        end
!
!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++




!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!        subroutine writedat
!
!        write spectral data to output files
!
!        local parameters: 

      subroutine writedat(recno,h,hsig,dx,ix,dwny,skew,asym,iloc)        

        use b2dvars, only: es,nsg,nwny,sg,wny
        
        real h(10001),hsig,dx,dwny
        integer recno,ix,iloc
        
        write (30,rec=recno) ((es(1,i,j), i=1,nsg), j=-nwny,nwny)

        print *,' location ',iloc,':   ',  &
                     ' x = ',float(ix+1)*dx,' m,   ',  &
                     ' h = ',h(ix+2),' m,   ',  &
                     ' Hs = ',hsig,' m'

        write (18,*) skew, asym

        return
        end
!
!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

